use crate::mathml_renderer::ast::Node;
use crate::mathml_renderer::attribute::{
    FracAttr, MathSpacing, MathVariant, OpAttr, RowAttr, Size, Style, TextTransform,
};
use crate::mathml_renderer::symbol::{self, Rel};

use super::predefined;
use super::specifications::LaTeXUnit;
use super::token::{NodeRef, Token};

static COMMANDS: phf::Map<&'static str, Token> = phf::phf_map! {
    " " => Token::NonBreakingSpace,
    "!" => Token::Space(LaTeXUnit::Mu.length_with_unit(-3.0)),
    "#" => Token::Letter(symbol::NUMBER_SIGN),
    "$" => Token::Letter(symbol::DOLLAR_SIGN),
    "%" => Token::Letter(symbol::PERCENT_SIGN),
    "&" => Token::OpAmpersand,
    "," => Token::Space(LaTeXUnit::Mu.length_with_unit(3.0)),
    ":" => Token::Space(LaTeXUnit::Mu.length_with_unit(4.0)),
    ";" => Token::Space(LaTeXUnit::Mu.length_with_unit(5.0)),
    ">" => Token::Space(LaTeXUnit::Mu.length_with_unit(4.0)),
    "Alpha" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_ALPHA),
    "And" => Token::OpAmpersand,
    "Angstrom" => Token::Letter(symbol::ANGSTROM_SIGN),
    "Bbbk" => Token::Letter(TextTransform::DoubleStruck.transform('k', false)),
    "Beta" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_BETA),
    "Big" => Token::Big(Size::Scale2),
    "Bigg" => Token::Big(Size::Scale4),
    "Biggl" => Token::Big(Size::Scale4),
    "Biggr" => Token::Big(Size::Scale4),
    "Bigl" => Token::Big(Size::Scale2),
    "Bigr" => Token::Big(Size::Scale2),
    "Box" => Token::Letter(symbol::WHITE_MEDIUM_SQUARE),
    "Bumpeq" => Token::Relation(symbol::GEOMETRICALLY_EQUIVALENT_TO),
    "Cap" => Token::BinaryOp(symbol::DOUBLE_INTERSECTION),
    "Chi" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_CHI),
    "Colon" => Token::Relation(symbol::PROPORTION),
    "Cup" => Token::BinaryOp(symbol::DOUBLE_UNION),
    "Dagger" => Token::Letter(symbol::DOUBLE_DAGGER),
    "Delta" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_DELTA),
    "Diamond" => Token::Letter(symbol::LOZENGE),
    "Doteq" => Token::Relation(symbol::GEOMETRICALLY_EQUAL_TO),
    "Downarrow" => Token::Delimiter(symbol::DOWNWARDS_DOUBLE_ARROW),
    "Epsilon" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_EPSILON),
    "Eta" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_ETA),
    "Finv" => Token::Letter(symbol::TURNED_CAPITAL_F),
    "Game" => Token::Letter(symbol::TURNED_SANS_SERIF_CAPITAL_G),
    "Gamma" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_GAMMA),
    "Im" => Token::Letter(symbol::BLACK_LETTER_CAPITAL_I),
    "Iota" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_IOTA),
    "Join" => Token::Relation(symbol::BOWTIE),
    "Kappa" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_KAPPA),
    "Lambda" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_LAMBDA),
    "Leftarrow" => Token::Relation(symbol::LEFTWARDS_DOUBLE_ARROW),
    "Leftrightarrow" => Token::Relation(symbol::LEFT_RIGHT_DOUBLE_ARROW),
    "Lleftarrow" => Token::Relation(symbol::LEFTWARDS_TRIPLE_ARROW),
    "Longleftarrow" => Token::Relation(symbol::LONG_LEFTWARDS_DOUBLE_ARROW),
    "Longleftrightarrow" => Token::Relation(symbol::LONG_LEFT_RIGHT_DOUBLE_ARROW),
    "Longrightarrow" => Token::Relation(symbol::LONG_RIGHTWARDS_DOUBLE_ARROW),
    "Lsh" => Token::Relation(symbol::UPWARDS_ARROW_WITH_TIP_LEFTWARDS),
    "Mu" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_MU),
    "Nu" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_NU),
    "Omega" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_OMEGA),
    "Omicron" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_OMICRON),
    "P" => Token::Letter(symbol::PILCROW_SIGN),
    "Phi" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_PHI),
    "Pi" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_PI),
    "Pr" => Token::PseudoOperator("Pr"),
    "Psi" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_PSI),
    "RR" => Token::Letter(symbol::DOUBLE_STRUCK_CAPITAL_R),
    "Re" => Token::Letter(symbol::BLACK_LETTER_CAPITAL_R),
    "Rho" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_RHO),
    "Rightarrow" => Token::Relation(symbol::RIGHTWARDS_DOUBLE_ARROW),
    "Rrightarrow" => Token::Relation(symbol::RIGHTWARDS_TRIPLE_ARROW),
    "Rsh" => Token::Relation(symbol::UPWARDS_ARROW_WITH_TIP_RIGHTWARDS),
    "S" => Token::Letter(symbol::SECTION_SIGN),
    "Sigma" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_SIGMA),
    "Subset" => Token::Relation(symbol::DOUBLE_SUBSET),
    "Supset" => Token::Relation(symbol::DOUBLE_SUPERSET),
    "Tau" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_TAU),
    "Theta" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_THETA),
    "Uparrow" => Token::Delimiter(symbol::UPWARDS_DOUBLE_ARROW),
    "Updownarrow" => Token::Delimiter(symbol::UP_DOWN_DOUBLE_ARROW),
    "Upsilon" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_UPSILON),
    "VDash" => Token::Relation(symbol::DOUBLE_VERTICAL_BAR_DOUBLE_RIGHT_TURNSTILE),
    "Vdash" => Token::Relation(symbol::FORCES),
    "Vert" => Token::Delimiter(symbol::DOUBLE_VERTICAL_LINE),
    "Vvdash" => Token::Relation(symbol::TRIPLE_VERTICAL_BAR_RIGHT_TURNSTILE),
    "Xi" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_XI),
    "Yleft" => Token::Relation(symbol::LEFTWARDS_ARROW_TAIL),
    "Yright" => Token::Relation(symbol::RIGHTWARDS_ARROW_TAIL),
    "Zeta" => Token::UprightLetter(symbol::GREEK_CAPITAL_LETTER_ZETA),
    "\\" => Token::NewLine,
    "_" => Token::Letter(symbol::LOW_LINE.as_op().as_char()),
    "acute" => Token::OverUnder(symbol::ACUTE_ACCENT, true, None),
    "aleph" => Token::Letter(symbol::ALEF_SYMBOL),
    "alpha" => Token::Letter(symbol::GREEK_SMALL_LETTER_ALPHA),
    "amalg" => Token::Relation(symbol::AMALGAMATION_OR_COPRODUCT),
    "angle" => Token::Letter(symbol::ANGLE),
    "approx" => Token::Relation(symbol::ALMOST_EQUAL_TO),
    "approxeq" => Token::Relation(symbol::ALMOST_EQUAL_OR_EQUAL_TO),
    "arccos" => Token::PseudoOperator("arccos"),
    "arceq" => Token::Relation(symbol::CORRESPONDS_TO), // from "stix"
    "arcsin" => Token::PseudoOperator("arcsin"),
    "arctan" => Token::PseudoOperator("arctan"),
    "arg" => Token::PseudoOperator("arg"),
    "argmax" => Token::PseudoOperatorLimits("arg\u{2009}max"),
    "argmin" => Token::PseudoOperatorLimits("arg\u{2009}min"),
    "ascnode" => Token::Letter(symbol::ASCENDING_NODE),
    "ast" => Token::Relation(symbol::ASTERISK_OPERATOR),
    "astrosun" => Token::Letter(symbol::SUN),
    "asymp" => Token::Relation(symbol::EQUIVALENT_TO),
    "awint" => Token::Integral(symbol::ANTICLOCKWISE_INTEGRATION),
    "backdprime" => Token::Ord(symbol::REVERSED_DOUBLE_PRIME),
    "backepsilon" => Token::Letter(symbol::GREEK_REVERSED_LUNATE_EPSILON_SYMBOL),
    "backprime" => Token::Ord(symbol::REVERSED_PRIME),
    "backsim" => Token::Relation(symbol::REVERSED_TILDE),
    "backsimeq" => Token::Relation(symbol::REVERSED_TILDE_EQUALS),
    "backslash" => Token::Delimiter(symbol::REVERSE_SOLIDUS),
    "backtrprime" => Token::Ord(symbol::REVERSED_TRIPLE_PRIME),
    "bar" => Token::OverUnder(symbol::MACRON, true, Some(OpAttr::StretchyFalse)),
    "barwedge" => Token::BinaryOp(symbol::NAND),
    "because" => Token::Relation(symbol::BECAUSE),
    "begin" => Token::Begin,
    "beta" => Token::Letter(symbol::GREEK_SMALL_LETTER_BETA),
    "beth" => Token::Letter(symbol::BET_SYMBOL),
    "between" => Token::Relation(symbol::BETWEEN),
    "big" => Token::Big(Size::Scale1),
    "bigcap" => Token::BigOp(symbol::N_ARY_INTERSECTION),
    "bigcirc" => Token::Letter(symbol::LARGE_CIRCLE),
    "bigcup" => Token::BigOp(symbol::N_ARY_UNION),
    "bigcupdot" => Token::BigOp(symbol::N_ARY_UNION_OPERATOR_WITH_DOT),
    "bigdoublevee" => Token::BigOp(symbol::TWO_LOGICAL_OR_OPERATOR),
    "bigdoublewedge" => Token::BigOp(symbol::TWO_LOGICAL_AND_OPERATOR),
    "bigg" => Token::Big(Size::Scale3),
    "biggl" => Token::Big(Size::Scale3),
    "biggr" => Token::Big(Size::Scale3),
    "bigl" => Token::Big(Size::Scale1),
    "bigodot" => Token::BigOp(symbol::N_ARY_CIRCLED_DOT_OPERATOR),
    "bigoplus" => Token::BigOp(symbol::N_ARY_CIRCLED_PLUS_OPERATOR),
    "bigotimes" => Token::BigOp(symbol::N_ARY_CIRCLED_TIMES_OPERATOR),
    "bigr" => Token::Big(Size::Scale1),
    "bigsqcap" => Token::BigOp(symbol::N_ARY_SQUARE_INTERSECTION_OPERATOR),
    "bigsqcup" => Token::BigOp(symbol::N_ARY_SQUARE_UNION_OPERATOR),
    "bigstar" => Token::Letter(symbol::BLACK_STAR),
    "bigtimes" => Token::BigOp(symbol::N_ARY_TIMES_OPERATOR),
    "bigtriangledown" => Token::Letter(symbol::WHITE_DOWN_POINTING_TRIANGLE),
    "bigtriangleup" => Token::Letter(symbol::WHITE_UP_POINTING_TRIANGLE),
    "biguplus" => Token::BigOp(symbol::N_ARY_UNION_OPERATOR_WITH_PLUS),
    "bigvee" => Token::BigOp(symbol::N_ARY_LOGICAL_OR),
    "bigwedge" => Token::BigOp(symbol::N_ARY_LOGICAL_AND),
    "binom" => Token::Binom(None),
    "blacklozenge" => Token::Letter(symbol::BLACK_LOZENGE),
    "blacksquare" => Token::Letter(symbol::BLACK_SQUARE),
    "blacktriangle" => Token::Letter(symbol::BLACK_UP_POINTING_TRIANGLE),
    "blacktriangledown" => Token::Letter(symbol::BLACK_DOWN_POINTING_TRIANGLE),
    "blacktriangleleft" => Token::Letter(symbol::BLACK_LEFT_POINTING_TRIANGLE),
    "blacktriangleright" => Token::Letter(symbol::BLACK_RIGHT_POINTING_TRIANGLE),
    "bm" => Token::Transform(MathVariant::Transform(TextTransform::BoldItalic)),
    "bmod" => Token::CustomCmd(0, NodeRef::new(&Node::Row {
        nodes: &[
            &Node::Space(LaTeXUnit::Mu.length_with_unit(4.0)),
            &Node::Text("mod"),
            &Node::Space(LaTeXUnit::Mu.length_with_unit(4.0)),
        ],
        attr: RowAttr::None
    })),
    "boldsymbol" => Token::Transform(MathVariant::Transform(TextTransform::BoldItalic)),
    "bot" => Token::Letter(symbol::UP_TACK),
    "botdoteq" => Token::Relation(symbol::EQUALS_SIGN_WITH_DOT_BELOW),
    "boxbox" => Token::Relation(symbol::SQUARED_SQUARE),
    "boxbslash" => Token::Relation(symbol::SQUARED_FALLING_DIAGONAL_SLASH),
    "boxdot" => Token::BinaryOp(symbol::SQUARED_DOT_OPERATOR),
    "boxminus" => Token::BinaryOp(symbol::SQUARED_MINUS),
    "boxplus" => Token::BinaryOp(symbol::SQUARED_PLUS),
    "boxslash" => Token::Relation(symbol::SQUARED_RISING_DIAGONAL_SLASH),
    "boxtimes" => Token::BinaryOp(symbol::SQUARED_TIMES),
    "breve" => Token::OverUnder(symbol::BREVE, true, None),
    "bullet" => Token::Relation(symbol::BULLET_OPERATOR),
    "bumpeq" => Token::Relation(symbol::DIFFERENCE_BETWEEN),
    "cap" => Token::BinaryOp(symbol::INTERSECTION),
    "cdot" => Token::BinaryOp(symbol::MIDDLE_DOT),
    "cdots" => Token::CustomCmd(0, NodeRef::new(&Node::Row {
        nodes: &[
            &Node::Operator {
                op:symbol::MIDDLE_DOT.as_op(),
                attr: None,
                left: None,
                right: None,
            },
            &Node::Operator {
                op:symbol::MIDDLE_DOT.as_op(),
                attr: None,
                left: Some(MathSpacing::Zero),
                right: Some(MathSpacing::Zero),
            },
            &Node::Operator {
                op:symbol::MIDDLE_DOT.as_op(),
                attr: None,
                left: None,
                right: None,
            },
        ],
        attr: RowAttr::None
    })),
    "centerdot" => Token::Relation(symbol::BULLET_OPERATOR),
    "cfrac" => Token::Frac(Some(FracAttr::CFracStyle)),
    "check" => Token::OverUnder(symbol::CARON, true, Some(OpAttr::StretchyFalse)),
    "checkmark" => Token::Letter('✓'),
    "chi" => Token::Letter(symbol::GREEK_SMALL_LETTER_CHI),
    "circ" => Token::Relation(symbol::RING_OPERATOR),
    "circeq" => Token::Relation(symbol::RING_EQUAL_TO),
    "circlearrowleft" => Token::Relation(symbol::ANTICLOCKWISE_OPEN_CIRCLE_ARROW),
    "circlearrowright" => Token::Relation(symbol::CLOCKWISE_OPEN_CIRCLE_ARROW),
    "circledR" => Token::Letter(symbol::CIRCLED_LATIN_CAPITAL_LETTER_R),
    "circledS" => Token::Letter(symbol::CIRCLED_LATIN_CAPITAL_LETTER_S),
    "circledast" => Token::BinaryOp(symbol::CIRCLED_ASTERISK_OPERATOR),
    "circledcirc" => Token::BinaryOp(symbol::CIRCLED_RING_OPERATOR),
    "circleddash" => Token::BinaryOp(symbol::CIRCLED_DASH),
    "cirfnint" => Token::Integral(symbol::CIRCULATION_FUNCTION),
    "clubsuit" => Token::Letter(symbol::BLACK_CLUB_SUIT),
    "colon" => Token::Letter(symbol::COLON.as_op().as_char()),
    "coloneq" => Token::Relation(symbol::COLON_EQUALS),
    "coloneqq" => Token::Relation(symbol::COLON_EQUALS),
    "color" => Token::Color,
    "complement" => Token::Letter(symbol::COMPLEMENT.as_op().as_char()), // should be Ord, but FF has a bug
    "cong" => Token::Relation(symbol::APPROXIMATELY_EQUAL_TO),
    "coprod" => Token::BigOp(symbol::N_ARY_COPRODUCT),
    "copyright" => Token::Letter(symbol::COPYRIGHT_SIGN),
    "cos" => Token::PseudoOperator("cos"),
    "cosh" => Token::PseudoOperator("cosh"),
    "cot" => Token::PseudoOperator("cot"),
    "coth" => Token::PseudoOperator("coth"),
    "csc" => Token::PseudoOperator("csc"),
    "cup" => Token::BinaryOp(symbol::UNION),
    "curlyeqprec" => Token::Relation(symbol::EQUAL_TO_OR_PRECEDES),
    "curlyeqsucc" => Token::Relation(symbol::EQUAL_TO_OR_SUCCEEDS),
    "curlyvee" => Token::BinaryOp(symbol::CURLY_LOGICAL_OR),
    "curlywedge" => Token::BinaryOp(symbol::CURLY_LOGICAL_AND),
    "curvearrowleft" => Token::Relation(symbol::ANTICLOCKWISE_TOP_SEMICIRCLE_ARROW),
    "curvearrowright" => Token::Relation(symbol::CLOCKWISE_TOP_SEMICIRCLE_ARROW),
    "dag" => Token::Letter(symbol::DAGGER),
    "dagger" => Token::Letter(symbol::DAGGER),
    "daleth" => Token::Letter(symbol::DALET_SYMBOL),
    "dashcolon" => Token::Relation(symbol::EXCESS),
    "dashv" => Token::Relation(symbol::LEFT_TACK),
    "dbinom" => Token::Binom(Some(FracAttr::DisplayStyleTrue)),
    "dblcolon" => Token::Relation(symbol::PROPORTION),
    "ddag" => Token::Letter(symbol::DOUBLE_DAGGER),
    "ddagger" => Token::Letter(symbol::DOUBLE_DAGGER),
    "ddddot" => Token::OverUnder(symbol::COMBINING_FOUR_DOTS_ABOVE, true, Some(OpAttr::StretchyFalse)),
    "dddot" => Token::OverUnder(symbol::COMBINING_THREE_DOTS_ABOVE, true, Some(OpAttr::StretchyFalse)),
    "ddot" => Token::OverUnder(symbol::DIAERESIS, true, Some(OpAttr::StretchyFalse)),
    "ddots" => Token::Relation(symbol::DOWN_RIGHT_DIAGONAL_ELLIPSIS),
    "deg" => Token::PseudoOperator("deg"),
    "delta" => Token::Letter(symbol::GREEK_SMALL_LETTER_DELTA),
    "det" => Token::PseudoOperator("det"),
    "dfrac" => Token::Frac(Some(FracAttr::DisplayStyleTrue)),
    "diamond" => Token::BinaryOp(symbol::DIAMOND_OPERATOR),
    "diamondsuit" => Token::Letter(symbol::WHITE_DIAMOND_SUIT),
    "digamma" => Token::Letter(symbol::GREEK_SMALL_LETTER_DIGAMMA),
    "dim" => Token::PseudoOperator("dim"),
    "displaystyle" => Token::Style(Style::Display),
    "div" => Token::BinaryOp(symbol::DIVISION_SIGN),
    "divideontimes" => Token::BinaryOp(symbol::DIVISION_TIMES),
    "dot" => Token::OverUnder(symbol::DOT_ABOVE, true, None),
    "doteq" => Token::Relation(symbol::APPROACHES_THE_LIMIT),
    "doteqdot" => Token::Relation(symbol::GEOMETRICALLY_EQUAL_TO),
    "dotplus" => Token::BinaryOp(symbol::DOT_PLUS),
    "dots" => Token::CustomCmd(0, NodeRef::new(&predefined::DOTS)),
    "dotsminusdots" => Token::Relation(symbol::GEOMETRIC_PROPORTION),
    "doublebarwedge" => Token::Relation(symbol::LOGICAL_AND_WITH_DOUBLE_OVERBAR),
    "downarrow" => Token::Delimiter(symbol::DOWNWARDS_ARROW),
    "downdownarrows" => Token::Relation(symbol::DOWNWARDS_PAIRED_ARROWS),
    "downharpoonleft" => Token::Relation(symbol::DOWNWARDS_HARPOON_WITH_BARB_LEFTWARDS),
    "downharpoonright" => Token::Relation(symbol::DOWNWARDS_HARPOON_WITH_BARB_RIGHTWARDS),
    "dprime" => Token::Ord(symbol::DOUBLE_PRIME),
    "earth" => Token::Letter(symbol::EARTH),
    "ell" => Token::Letter(symbol::SCRIPT_SMALL_L),
    "empty" => Token::CustomCmd(0, NodeRef::new(&Node::IdentifierStr("∅\u{FE00}"))), // with Unicode variation selector
    "emptyset" => Token::CustomCmd(0, NodeRef::new(&Node::IdentifierStr("∅\u{FE00}"))), // with Unicode variation selector
    "end" => Token::End,
    "epsilon" => Token::Letter(symbol::GREEK_LUNATE_EPSILON_SYMBOL),
    "eqcirc" => Token::Relation(symbol::RING_IN_EQUAL_TO),
    "eqcolon" => Token::Relation(symbol::EQUALS_COLON),
    "eqdef" => Token::Relation(symbol::EQUAL_TO_BY_DEFINITION), // from "stix"
    "eqqcolon" => Token::Relation(symbol::EQUALS_COLON),
    "eqsim" => Token::Relation(symbol::MINUS_TILDE),
    "eqslantgtr" => Token::Relation(symbol::SLANTED_EQUAL_TO_OR_GREATER_THAN),
    "eqslantless" => Token::Relation(symbol::SLANTED_EQUAL_TO_OR_LESS_THAN),
    "equiv" => Token::Relation(symbol::IDENTICAL_TO),
    "erf" => Token::PseudoOperator("erf"),
    "erfc" => Token::PseudoOperator("erfc"),
    "eta" => Token::Letter(symbol::GREEK_SMALL_LETTER_ETA),
    "eth" => Token::Letter(symbol::LATIN_SMALL_LETTER_ETH),
    "euro" => Token::Letter('€'),
    "exists" => Token::Ord(symbol::THERE_EXISTS),
    "exp" => Token::PseudoOperator("exp"),
    "fallingdotseq" => Token::Relation(symbol::APPROXIMATELY_EQUAL_TO_OR_THE_IMAGE_OF),
    "fcmp" => Token::Relation(symbol::Z_NOTATION_SCHEMA_COMPOSITION),
    "fint" => Token::Integral(symbol::INTEGRAL_AVERAGE_WITH_SLASH),
    "flat" => Token::Letter('♭'),
    "forall" => Token::Ord(symbol::FOR_ALL),
    "frac" => Token::Frac(None),
    "frown" => Token::Relation(symbol::FROWN),
    "gamma" => Token::Letter(symbol::GREEK_SMALL_LETTER_GAMMA),
    "gcd" => Token::PseudoOperator("gcd"),
    "ge" => Token::Relation(symbol::GREATER_THAN_OR_EQUAL_TO),
    "genfrac" => Token::Genfrac,
    "geq" => Token::Relation(symbol::GREATER_THAN_OR_EQUAL_TO),
    "geqq" => Token::Relation(symbol::GREATER_THAN_OVER_EQUAL_TO),
    "geqslant" => Token::Relation(symbol::GREATER_THAN_OR_SLANTED_EQUAL_TO),
    "gets" => Token::Relation(symbol::LEFTWARDS_ARROW),
    "gg" => Token::Relation(symbol::MUCH_GREATER_THAN),
    "gimel" => Token::Letter(symbol::GIMEL_SYMBOL),
    "gnapprox" => Token::Relation(symbol::GREATER_THAN_AND_NOT_APPROXIMATE),
    "gneq" => Token::Relation(symbol::GREATER_THAN_AND_SINGLE_LINE_NOT_EQUAL_TO),
    "gneqq" => Token::Relation(symbol::GREATER_THAN_BUT_NOT_EQUAL_TO),
    "gnsim" => Token::Relation(symbol::GREATER_THAN_BUT_NOT_EQUIVALENT_TO),
    "grave" => Token::OverUnder(symbol::GRAVE_ACCENT, true, None),
    "gt" => Token::OpGreaterThan,
    "gtrapprox" => Token::Relation(symbol::GREATER_THAN_OR_APPROXIMATE),
    "gtreqless" => Token::Relation(symbol::GREATER_THAN_EQUAL_TO_OR_LESS_THAN),
    "gtreqqless" => Token::Relation(symbol::GREATER_THAN_ABOVE_DOUBLE_LINE_EQUAL_ABOVE_LESS_THAN),
    "gtrless" => Token::Relation(symbol::GREATER_THAN_OR_LESS_THAN),
    "gtrsim" => Token::Relation(symbol::GREATER_THAN_OR_EQUIVALENT_TO),
    "hat" => Token::OverUnder(symbol::COMBINING_CIRCUMFLEX_ACCENT, true, Some(OpAttr::StretchyFalse)),
    "hbar" => Token::Letter(symbol::PLANCK_CONSTANT_OVER_TWO_PI),
    "heartsuit" => Token::Letter(symbol::WHITE_HEART_SUIT),
    "hom" => Token::PseudoOperator("hom"),
    "hookleftarrow" => Token::Relation(symbol::LEFTWARDS_ARROW_WITH_HOOK),
    "hookrightarrow" => Token::Relation(symbol::RIGHTWARDS_ARROW_WITH_HOOK),
    "hslash" => Token::Letter(symbol::PLANCK_CONSTANT_OVER_TWO_PI),
    "hspace" => Token::CustomSpace,
    "iff" => Token::Relation(symbol::LONG_LEFT_RIGHT_DOUBLE_ARROW),
    "iiiint" => Token::Integral(symbol::QUADRUPLE_INTEGRAL_OPERATOR),
    "iiint" => Token::Integral(symbol::TRIPLE_INTEGRAL),
    "iint" => Token::Integral(symbol::DOUBLE_INTEGRAL),
    "imath" => Token::Letter(symbol::LATIN_SMALL_LETTER_DOTLESS_I),
    "impliedby" => Token::Relation(symbol::LONG_LEFTWARDS_DOUBLE_ARROW),
    "implies" => Token::Relation(symbol::LONG_RIGHTWARDS_DOUBLE_ARROW),
    "in" => Token::Relation(symbol::ELEMENT_OF),
    "inf" => Token::PseudoOperatorLimits("inf"),
    "infty" => Token::Letter(symbol::INFINITY),
    "int" => Token::Integral(symbol::INTEGRAL),
    "intBar" => Token::Integral(symbol::INTEGRAL_WITH_DOUBLE_STROKE),
    "intbar" => Token::Integral(symbol::FINITE_PARTL_INTEGRAL),
    "intclockwise" => Token::Integral(symbol::CLOCKWISE_INTEGRAL),
    "intercal" => Token::Relation(symbol::INTERCALATE),
    "iota" => Token::Letter(symbol::GREEK_SMALL_LETTER_IOTA),
    "jmath" => Token::Letter(symbol::LATIN_SMALL_LETTER_DOTLESS_J),
    "jupiter" => Token::Letter(symbol::JUPITER),
    "kappa" => Token::Letter(symbol::GREEK_SMALL_LETTER_KAPPA),
    "ker" => Token::PseudoOperator("ker"),
    "kernelcontraction" => Token::Relation(symbol::HOMOTHETIC),
    "lBrace" => Token::Delimiter(symbol::LEFT_WHITE_CURLY_BRACKET),
    "lVert" => Token::Delimiter(symbol::DOUBLE_VERTICAL_LINE),
    "lambda" => Token::Letter(symbol::GREEK_SMALL_LETTER_LAMBDA),
    "land" => Token::BinaryOp(symbol::LOGICAL_AND),
    "langle" => Token::Delimiter(symbol::MATHEMATICAL_LEFT_ANGLE_BRACKET),
    "lbrace" => Token::Delimiter(symbol::LEFT_CURLY_BRACKET),
    "lbrack" => Token::Delimiter(symbol::LEFT_SQUARE_BRACKET),
    "lceil" => Token::Delimiter(symbol::LEFT_CEILING),
    "ldots" => Token::CustomCmd(0, NodeRef::new(&predefined::DOTS)),
    "le" => Token::Relation(symbol::LESS_THAN_OR_EQUAL_TO),
    "left" => Token::Left,
    "leftarrow" => Token::Relation(symbol::LEFTWARDS_ARROW),
    "leftarrowtail" => Token::Relation(symbol::LEFTWARDS_ARROW_WITH_TAIL),
    "leftharpoondown" => Token::Relation(symbol::LEFTWARDS_HARPOON_WITH_BARB_DOWNWARDS),
    "leftharpoonup" => Token::Relation(symbol::LEFTWARDS_HARPOON_WITH_BARB_UPWARDS),
    "leftleftarrows" => Token::Relation(symbol::LEFTWARDS_PAIRED_ARROWS),
    "leftrightarrow" => Token::Relation(symbol::LEFT_RIGHT_ARROW),
    "leftrightarrows" => Token::Relation(symbol::LEFTWARDS_ARROW_OVER_RIGHTWARDS_ARROW),
    "leftrightharpoons" => Token::Relation(symbol::LEFTWARDS_HARPOON_OVER_RIGHTWARDS_HARPOON),
    "leftrightsquigarrow" => Token::Relation(symbol::LEFT_RIGHT_WAVE_ARROW),
    "leftthreetimes" => Token::Relation(symbol::LEFT_SEMIDIRECT_PRODUCT),
    "leq" => Token::Relation(symbol::LESS_THAN_OR_EQUAL_TO),
    "leqq" => Token::Relation(symbol::LESS_THAN_OVER_EQUAL_TO),
    "leqslant" => Token::Relation(symbol::LESS_THAN_OR_SLANTED_EQUAL_TO),
    "lessapprox" => Token::Relation(symbol::LESS_THAN_OR_APPROXIMATE),
    "lessdot" => Token::Relation(symbol::LESS_THAN_WITH_DOT),
    "lesseqgtr" => Token::Relation(symbol::LESS_THAN_EQUAL_TO_OR_GREATER_THAN),
    "lesseqqgtr" => Token::Relation(symbol::LESS_THAN_ABOVE_DOUBLE_LINE_EQUAL_ABOVE_GREATER_THAN),
    "lessgtr" => Token::Relation(symbol::LESS_THAN_OR_GREATER_THAN),
    "lesssim" => Token::Relation(symbol::LESS_THAN_OR_EQUIVALENT_TO),
    "lfloor" => Token::Delimiter(symbol::LEFT_FLOOR),
    "lg" => Token::PseudoOperator("lg"),
    "lgroup" => Token::Delimiter(symbol::MATHEMATICAL_LEFT_FLATTENED_PARENTHESIS),
    "lhd" => Token::Relation(symbol::NORMAL_SUBGROUP_OF),
    "lightning" => Token::Relation(symbol::DOWNWARDS_ZIGZAG_ARROW),
    "lim" => Token::PseudoOperatorLimits("lim"),
    "liminf" => Token::PseudoOperatorLimits("lim\u{2009}inf"),
    "limits" => Token::Limits,
    "limsup" => Token::PseudoOperatorLimits("lim\u{2009}sup"),
    "ll" => Token::Relation(symbol::MUCH_LESS_THAN),
    "llangle" => Token::Delimiter(symbol::Z_NOTATION_LEFT_BINDING_BRACKET),
    "llbracket" => Token::Delimiter(symbol::MATHEMATICAL_LEFT_WHITE_SQUARE_BRACKET),
    "llcorner" => Token::Letter(symbol::BOTTOM_LEFT_CORNER),
    "lll" => Token::Relation(symbol::VERY_MUCH_LESS_THAN),
    "llparenthesis" => Token::Delimiter(symbol::Z_NOTATION_LEFT_IMAGE_BRACKET),
    "ln" => Token::PseudoOperator("ln"),
    "lnapprox" => Token::Relation(symbol::LESS_THAN_AND_NOT_APPROXIMATE),
    "lneq" => Token::Relation(symbol::LESS_THAN_AND_SINGLE_LINE_NOT_EQUAL_TO),
    "lneqq" => Token::Relation(symbol::LESS_THAN_BUT_NOT_EQUAL_TO),
    "lnot" => Token::Ord(symbol::NOT_SIGN),
    "lnsim" => Token::Relation(symbol::LESS_THAN_BUT_NOT_EQUIVALENT_TO),
    "log" => Token::PseudoOperator("log"),
    "longleftarrow" => Token::Relation(symbol::LONG_LEFTWARDS_ARROW),
    "longleftrightarrow" => Token::Relation(symbol::LONG_LEFT_RIGHT_ARROW),
    "longmapsto" => Token::Relation(symbol::LONG_RIGHTWARDS_ARROW_FROM_BAR),
    "longrightarrow" => Token::Relation(symbol::LONG_RIGHTWARDS_ARROW),
    "looparrowleft" => Token::Relation(symbol::LEFTWARDS_ARROW_WITH_LOOP),
    "looparrowright" => Token::Relation(symbol::RIGHTWARDS_ARROW_WITH_LOOP),
    "lor" => Token::BinaryOp(symbol::LOGICAL_OR),
    "lozenge" => Token::Letter(symbol::LOZENGE),
    "lrcorner" => Token::Letter(symbol::BOTTOM_RIGHT_CORNER),
    "lt" => Token::OpLessThan,
    "ltimes" => Token::Relation(symbol::LEFT_NORMAL_FACTOR_SEMIDIRECT_PRODUCT),
    "lvert" => Token::Delimiter(symbol::VERTICAL_LINE),
    "maltese" => Token::Letter(symbol::MALTESE_CROSS),
    "mapsto" => Token::Relation(symbol::RIGHTWARDS_ARROW_FROM_BAR),
    "mars" => Token::Letter(symbol::MALE_SIGN),
    "mathbb" => Token::Transform(MathVariant::Transform(TextTransform::DoubleStruck)),
    "mathbf" => Token::Transform(MathVariant::Transform(TextTransform::Bold)),
    "mathcal" => Token::Transform(MathVariant::Transform(TextTransform::ScriptChancery)),
    "mathfrak" => Token::Transform(MathVariant::Transform(TextTransform::Fraktur)),
    "mathit" => Token::Transform(MathVariant::Transform(TextTransform::Italic)),
    "mathrm" => Token::Transform(MathVariant::Normal),
    "mathscr" => Token::Transform(MathVariant::Transform(TextTransform::ScriptRoundhand)),
    "mathsf" => Token::Transform(MathVariant::Transform(TextTransform::SansSerif)),
    "mathstrut" => Token::HardcodedMathML(r#"<mpadded width="0" style="visibility:hidden"><mo stretchy="false">(</mo></mpadded>"#),
    "mathtt" => Token::Transform(MathVariant::Transform(TextTransform::Monospace)),
    "max" => Token::PseudoOperatorLimits("max"),
    "measeq" => Token::Relation(symbol::MEASURED_BY), // from "stix"
    "measuredangle" => Token::Letter(symbol::MEASURED_ANGLE),
    "mercury" => Token::Letter(symbol::MERCURY),
    "mho" => Token::Letter(symbol::INVERTED_OHM_SIGN),
    "mid" => Token::Relation(symbol::DIVIDES),
    "middle" => Token::Middle,
    "min" => Token::PseudoOperatorLimits("min"),
    "mod" => Token::CustomCmd(1, NodeRef::new(&predefined::MOD)),
    "models" => Token::Relation(symbol::TRUE),
    "mp" => Token::BinaryOp(symbol::MINUS_OR_PLUS_SIGN),
    "mspace" => Token::CustomSpace,
    "mu" => Token::Letter(symbol::GREEK_SMALL_LETTER_MU),
    "multimap" => Token::Relation(symbol::MULTIMAP),
    "nLeftarrow" => Token::Relation(symbol::LEFTWARDS_DOUBLE_ARROW_WITH_STROKE),
    "nLeftrightarrow" => Token::Relation(symbol::LEFT_RIGHT_DOUBLE_ARROW_WITH_STROKE),
    "nRightarrow" => Token::Relation(symbol::RIGHTWARDS_DOUBLE_ARROW_WITH_STROKE),
    "nVDash" => Token::Relation(symbol::NEGATED_DOUBLE_VERTICAL_BAR_DOUBLE_RIGHT_TURNSTILE),
    "nVdash" => Token::Relation(symbol::DOES_NOT_FORCE),
    "nabla" => Token::UprightLetter(symbol::NABLA),
    "natural" => Token::Letter('♮'),
    "ne" => Token::Relation(symbol::NOT_EQUAL_TO),
    "nearrow" => Token::Relation(symbol::NORTH_EAST_ARROW),
    "neg" => Token::Ord(symbol::NOT_SIGN),
    "neptune" => Token::Letter(symbol::NEPTUNE),
    "neq" => Token::Relation(symbol::NOT_EQUAL_TO),
    "nequiv" => Token::Relation(symbol::NOT_IDENTICAL_TO),
    "nexists" => Token::Ord(symbol::THERE_DOES_NOT_EXIST),
    "ngeq" => Token::Relation(symbol::NEITHER_GREATER_THAN_NOR_EQUAL_TO),
    "ngtr" => Token::Relation(symbol::NOT_GREATER_THAN),
    "ngtrless" => Token::Relation(symbol::NEITHER_GREATER_THAN_NOR_LESS_THAN),
    "ngtrsim" => Token::Relation(symbol::NEITHER_GREATER_THAN_NOR_EQUIVALENT_TO),
    "ni" => Token::Relation(symbol::CONTAINS_AS_MEMBER),
    "nleftarrow" => Token::Relation(symbol::LEFTWARDS_ARROW_WITH_STROKE),
    "nleftrightarrow" => Token::Relation(symbol::LEFT_RIGHT_ARROW_WITH_STROKE),
    "nleq" => Token::Relation(symbol::NEITHER_LESS_THAN_NOR_EQUAL_TO),
    "nless" => Token::Relation(symbol::NOT_LESS_THAN),
    "nlessgt" => Token::Relation(symbol::NEITHER_LESS_THAN_NOR_GREATER_THAN),
    "nlesssim" => Token::Relation(symbol::NEITHER_LESS_THAN_NOR_EQUIVALENT_TO),
    "nmid" => Token::Relation(symbol::DOES_NOT_DIVIDE),
    "not" => Token::Not,
    "notin" => Token::Relation(symbol::NOT_AN_ELEMENT_OF),
    "nparallel" => Token::Relation(symbol::NOT_PARALLEL_TO),
    "nprec" => Token::Relation(symbol::DOES_NOT_PRECEDE),
    "npreceq" => Token::Relation(symbol::DOES_NOT_PRECEDE_OR_EQUAL),
    "nrightarrow" => Token::Relation(symbol::RIGHTWARDS_ARROW_WITH_STROKE),
    "nsim" => Token::Relation(symbol::NOT_TILDE),
    "nsubset" => Token::Relation(symbol::NOT_A_SUBSET_OF),
    "nsubseteq" => Token::Relation(symbol::NEITHER_A_SUBSET_OF_NOR_EQUAL_TO),
    "nsucc" => Token::Relation(symbol::DOES_NOT_SUCCEED),
    "nsucceq" => Token::Relation(symbol::DOES_NOT_SUCCEED_OR_EQUAL),
    "nsupset" => Token::Relation(symbol::NOT_A_SUPERSET_OF),
    "nsupseteq" => Token::Relation(symbol::NEITHER_A_SUPERSET_OF_NOR_EQUAL_TO),
    "nu" => Token::Letter(symbol::GREEK_SMALL_LETTER_NU),
    "nvDash" => Token::Relation(symbol::NOT_TRUE),
    "nvdash" => Token::Relation(symbol::DOES_NOT_PROVE),
    "nwarrow" => Token::Relation(symbol::NORTH_WEST_ARROW),
    "odot" => Token::BinaryOp(symbol::CIRCLED_DOT_OPERATOR),
    "odv" => Token::CustomCmd(2, NodeRef::new(&predefined::ODV)),
    "oiiint" => Token::Integral(symbol::VOLUME_INTEGRAL),
    "oiint" => Token::Integral(symbol::SURFACE_INTEGRAL),
    "oint" => Token::Integral(symbol::CONTOUR_INTEGRAL),
    "ointctrclockwise" => Token::Integral(symbol::ANTICLOCKWISE_CONTOUR_INTEGRAL),
    "omega" => Token::Letter(symbol::GREEK_SMALL_LETTER_OMEGA),
    "omicron" => Token::Letter(symbol::GREEK_SMALL_LETTER_OMICRON),
    "ominus" => Token::BinaryOp(symbol::CIRCLED_MINUS),
    "operatorname" => Token::OperatorName,
    "oplus" => Token::BinaryOp(symbol::CIRCLED_PLUS),
    "oslash" => Token::BinaryOp(symbol::CIRCLED_DIVISION_SLASH),
    "otimes" => Token::BinaryOp(symbol::CIRCLED_TIMES),
    "overbrace" => Token::OverUnderBrace(symbol::TOP_CURLY_BRACKET, true),
    "overbracket" => Token::OverUnderBrace(symbol::TOP_SQUARE_BRACKET, true),
    "overleftarrow" => Token::OverUnder(symbol::LEFTWARDS_ARROW, true, None),
    "overline" => Token::OverUnder(symbol::OVERLINE, true, None),
    "overparen" => Token::OverUnderBrace(symbol::TOP_PARENTHESIS, true),
    "overrightarrow" => Token::OverUnder(symbol::RIGHTWARDS_ARROW, true, None),
    "overset" => Token::Overset,
    "parallel" => Token::Relation(symbol::PARALLEL_TO),
    "partial" => Token::Letter(symbol::PARTIAL_DIFFERENTIAL),
    "perp" => Token::Relation(symbol::PERPENDICULAR),
    "phi" => Token::Letter(symbol::GREEK_PHI_SYMBOL),
    "pi" => Token::Letter(symbol::GREEK_SMALL_LETTER_PI),
    "pitchfork" => Token::Relation(symbol::PITCHFORK),
    "pm" => Token::BinaryOp(symbol::PLUS_MINUS_SIGN),
    "pmod" => Token::CustomCmd(1, NodeRef::new(&predefined::PMOD)),
    "pounds" => Token::Letter('£'),
    "prec" => Token::Relation(symbol::PRECEDES),
    "precapprox" => Token::Relation(symbol::PRECEDES_ABOVE_ALMOST_EQUAL_TO),
    "preccurlyeq" => Token::Relation(symbol::PRECEDES_OR_EQUAL_TO),
    "preceq" => Token::Relation(symbol::PRECEDES_ABOVE_SINGLE_LINE_EQUALS_SIGN),
    "precnapprox" => Token::Relation(symbol::PRECEDES_ABOVE_NOT_ALMOST_EQUAL_TO),
    "precneqq" => Token::Relation(symbol::PRECEDES_ABOVE_NOT_EQUAL_TO),
    "precnsim" => Token::Relation(symbol::PRECEDES_BUT_NOT_EQUIVALENT_TO),
    "precsim" => Token::Relation(symbol::PRECEDES_OR_EQUIVALENT_TO),
    "prime" => Token::Ord(symbol::PRIME),
    "prod" => Token::BigOp(symbol::N_ARY_PRODUCT),
    "propto" => Token::Relation(symbol::PROPORTIONAL_TO),
    "psi" => Token::Letter(symbol::GREEK_SMALL_LETTER_PSI),
    "qprime" => Token::Ord(symbol::QUADRUPLE_PRIME),
    "qquad" => Token::Space(LaTeXUnit::Em.length_with_unit(2.0)),
    "quad" => Token::Space(LaTeXUnit::Em.length_with_unit(1.0)),
    "questeq" => Token::Relation(symbol::QUESTIONED_EQUAL_TO), // from "stix"
    "rBrace" => Token::Delimiter(symbol::RIGHT_WHITE_CURLY_BRACKET),
    "rVert" => Token::Delimiter(symbol::DOUBLE_VERTICAL_LINE),
    "rangle" => Token::Delimiter(symbol::MATHEMATICAL_RIGHT_ANGLE_BRACKET),
    "rbrace" => Token::Delimiter(symbol::RIGHT_CURLY_BRACKET),
    "rbrack" => Token::Delimiter(symbol::RIGHT_SQUARE_BRACKET),
    "rceil" => Token::Delimiter(symbol::RIGHT_CEILING),
    "rfloor" => Token::Delimiter(symbol::RIGHT_FLOOR),
    "rgroup" => Token::Delimiter(symbol::MATHEMATICAL_RIGHT_FLATTENED_PARENTHESIS),
    "rhd" => Token::Relation(symbol::CONTAINS_AS_NORMAL_SUBGROUP),
    "rho" => Token::Letter(symbol::GREEK_SMALL_LETTER_RHO),
    "right" => Token::Right,
    "rightarrow" => Token::Relation(symbol::RIGHTWARDS_ARROW),
    "rightarrowtail" => Token::Relation(symbol::RIGHTWARDS_ARROW_WITH_TAIL),
    "rightharpoondown" => Token::Relation(symbol::RIGHTWARDS_HARPOON_WITH_BARB_DOWNWARDS),
    "rightharpoonup" => Token::Relation(symbol::RIGHTWARDS_HARPOON_WITH_BARB_UPWARDS),
    "rightleftarrows" => Token::Relation(symbol::RIGHTWARDS_ARROW_OVER_LEFTWARDS_ARROW),
    "rightleftharpoons" => Token::Relation(symbol::RIGHTWARDS_HARPOON_OVER_LEFTWARDS_HARPOON),
    "rightrightarrows" => Token::Relation(symbol::RIGHTWARDS_PAIRED_ARROWS),
    "rightsquigarrow" => Token::Relation(symbol::RIGHTWARDS_SQUIGGLE_ARROW),
    "rightthreetimes" => Token::Relation(symbol::RIGHT_SEMIDIRECT_PRODUCT),
    "risingdotseq" => Token::Relation(symbol::IMAGE_OF_OR_APPROXIMATELY_EQUAL_TO),
    "rq" => Token::Letter('’'),
    "rrangle" => Token::Delimiter(symbol::Z_NOTATION_RIGHT_BINDING_BRACKET),
    "rrbracket" => Token::Delimiter(symbol::MATHEMATICAL_RIGHT_WHITE_SQUARE_BRACKET),
    "rrparenthesis" => Token::Delimiter(symbol::Z_NOTATION_RIGHT_IMAGE_BRACKET),
    "rtimes" => Token::Relation(symbol::RIGHT_NORMAL_FACTOR_SEMIDIRECT_PRODUCT),
    "rupee" => Token::Letter('₹'),
    "rvert" => Token::Delimiter(symbol::VERTICAL_LINE),
    "saturn" => Token::Letter(symbol::SATURN),
    "scriptscriptstyle" => Token::Style(Style::ScriptScript),
    "scriptstyle" => Token::Style(Style::Script),
    "searrow" => Token::Relation(symbol::SOUTH_EAST_ARROW),
    "sec" => Token::PseudoOperator("sec"),
    "setminus" => Token::Relation(symbol::SET_MINUS),
    "sgn" => Token::PseudoOperator("sgn"),
    "sharp" => Token::Letter('♯'),
    "sigma" => Token::Letter(symbol::GREEK_SMALL_LETTER_SIGMA),
    "sim" => Token::Relation(symbol::TILDE_OPERATOR),
    "simeq" => Token::Relation(symbol::ASYMPTOTICALLY_EQUAL_TO),
    "sin" => Token::PseudoOperator("sin"),
    "sinh" => Token::PseudoOperator("sinh"),
    "slashed" => Token::Slashed,
    "smallsetminus" => Token::Relation(symbol::SMALL_REVERSE_SOLIDUS),
    "smile" => Token::Relation(symbol::SMILE),
    "spadesuit" => Token::Letter(symbol::BLACK_SPADE_SUIT),
    "sphericalangle" => Token::Letter(symbol::SPHERICAL_ANGLE),
    "sqcap" => Token::BinaryOp(symbol::SQUARE_CAP),
    "sqcup" => Token::BinaryOp(symbol::SQUARE_CUP),
    "sqrt" => Token::Sqrt,
    "sqsubset" => Token::Relation(symbol::SQUARE_IMAGE_OF),
    "sqsubseteq" => Token::Relation(symbol::SQUARE_IMAGE_OF_OR_EQUAL_TO),
    "sqsupset" => Token::Relation(symbol::SQUARE_ORIGINAL_OF),
    "sqsupseteq" => Token::Relation(symbol::SQUARE_ORIGINAL_OF_OR_EQUAL_TO),
    "square" => Token::Letter(symbol::WHITE_SQUARE),
    "stackrel" => Token::Overset,
    "star" => Token::BinaryOp(symbol::STAR_OPERATOR),
    "stareq" => Token::Relation(symbol::STAR_EQUALS), // from "stix"
    "subset" => Token::Relation(symbol::SUBSET_OF),
    "subseteq" => Token::Relation(symbol::SUBSET_OF_OR_EQUAL_TO),
    "subsetneq" => Token::Relation(symbol::SUBSET_OF_WITH_NOT_EQUAL_TO),
    "subsetneqq" => Token::Relation(symbol::SUBSET_OF_ABOVE_NOT_EQUAL_TO),
    "succ" => Token::Relation(symbol::SUCCEEDS),
    "succapprox" => Token::Relation(symbol::SUCCEEDS_ABOVE_ALMOST_EQUAL_TO),
    "succcurlyeq" => Token::Relation(symbol::SUCCEEDS_OR_EQUAL_TO),
    "succeq" => Token::Relation(symbol::SUCCEEDS_ABOVE_SINGLE_LINE_EQUALS_SIGN),
    "succnapprox" => Token::Relation(symbol::SUCCEEDS_ABOVE_NOT_ALMOST_EQUAL_TO),
    "succneqq" => Token::Relation(symbol::SUCCEEDS_ABOVE_NOT_EQUAL_TO),
    "succnsim" => Token::Relation(symbol::SUCCEEDS_BUT_NOT_EQUIVALENT_TO),
    "succsim" => Token::Relation(symbol::SUCCEEDS_OR_EQUIVALENT_TO),
    "sum" => Token::BigOp(symbol::N_ARY_SUMMATION),
    "sumint" => Token::BigOp(symbol::SUMMATION_WITH_INTEGRAL),
    "sun" => Token::Letter(symbol::WHITE_SUN_WITH_RAYS),
    "sup" => Token::PseudoOperatorLimits("sup"),
    "supset" => Token::Relation(symbol::SUPERSET_OF),
    "supseteq" => Token::Relation(symbol::SUPERSET_OF_OR_EQUAL_TO),
    "supsetneq" => Token::Relation(symbol::SUPERSET_OF_WITH_NOT_EQUAL_TO),
    "supsetneqq" => Token::Relation(symbol::SUPERSET_OF_ABOVE_NOT_EQUAL_TO),
    "swarrow" => Token::Relation(symbol::SOUTH_WEST_ARROW),
    "symbf" => Token::Transform(MathVariant::Transform(TextTransform::BoldItalic)),
    "tan" => Token::PseudoOperator("tan"),
    "tanh" => Token::PseudoOperator("tanh"),
    "tau" => Token::Letter(symbol::GREEK_SMALL_LETTER_TAU),
    "tbinom" => Token::Binom(Some(FracAttr::DisplayStyleFalse)),
    "text" => Token::Text(None),
    "textbf" => Token::Text(Some(TextTransform::Bold)),
    "textit" => Token::Text(Some(TextTransform::Italic)),
    "textstyle" => Token::Style(Style::Text),
    "texttt" => Token::Text(Some(TextTransform::Monospace)),
    "tfrac" => Token::Frac(Some(FracAttr::DisplayStyleFalse)),
    "th" => Token::Letter(symbol::LATIN_SMALL_LETTER_THORN),
    "therefore" => Token::Relation(symbol::THEREFORE),
    "theta" => Token::Letter(symbol::GREEK_SMALL_LETTER_THETA),
    "tilde" => Token::OverUnder(symbol::COMBINING_TILDE, true, Some(OpAttr::StretchyFalse)),
    "times" => Token::BinaryOp(symbol::MULTIPLICATION_SIGN),
    "to" => Token::Relation(symbol::RIGHTWARDS_ARROW),
    "top" => Token::Letter(symbol::DOWN_TACK),
    "triangle" => Token::Letter(symbol::WHITE_UP_POINTING_TRIANGLE),
    "triangledown" => Token::Letter(symbol::WHITE_DOWN_POINTING_TRIANGLE),
    "triangleleft" => Token::Letter(symbol::WHITE_LEFT_POINTING_TRIANGLE),
    "triangleq" => Token::Relation(symbol::DELTA_EQUAL_TO),
    "triangleright" => Token::Letter(symbol::WHITE_RIGHT_POINTING_TRIANGLE),
    "trprime" => Token::Ord(symbol::TRIPLE_PRIME),
    "twoheadleftarrow" => Token::Relation(symbol::LEFTWARDS_TWO_HEADED_ARROW),
    "twoheadrightarrow" => Token::Relation(symbol::RIGHTWARDS_TWO_HEADED_ARROW),
    "ulcorner" => Token::Letter(symbol::TOP_LEFT_CORNER),
    "underbrace" => Token::OverUnderBrace(symbol::BOTTOM_CURLY_BRACKET, false),
    "underbracket" => Token::OverUnderBrace(symbol::BOTTOM_SQUARE_BRACKET, false),
    "underline" => Token::OverUnder(symbol::LOW_LINE, false, None),
    "underparen" => Token::OverUnderBrace(symbol::BOTTOM_PARENTHESIS, false),
    "underset" => Token::Underset,
    "unlhd" => Token::Relation(symbol::NORMAL_SUBGROUP_OF_OR_EQUAL_TO),
    "unrhd" => Token::Relation(symbol::CONTAINS_AS_NORMAL_SUBGROUP_OR_EQUAL_TO),
    "uparrow" => Token::Delimiter(symbol::UPWARDS_ARROW),
    "updownarrow" => Token::Delimiter(symbol::UP_DOWN_ARROW),
    "upharpoonleft" => Token::Relation(symbol::UPWARDS_HARPOON_WITH_BARB_LEFTWARDS),
    "upharpoonright" => Token::Relation(symbol::UPWARDS_HARPOON_WITH_BARB_RIGHTWARDS),
    "uplus" => Token::BinaryOp(symbol::MULTISET_UNION),
    "upsilon" => Token::Letter(symbol::GREEK_SMALL_LETTER_UPSILON),
    "upuparrows" => Token::Relation(symbol::UPWARDS_PAIRED_ARROWS),
    "uranus" => Token::Letter(symbol::URANUS),
    "urcorner" => Token::Letter(symbol::TOP_RIGHT_CORNER),
    "vDash" => Token::Relation(symbol::TRUE),
    "varDelta" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_DELTA), // italicized
    "varGamma" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_GAMMA), // italicized
    "varLambda" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_LAMBDA), // italicized
    "varOmega" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_OMEGA), // italicized
    "varPhi" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_PHI), // italicized
    "varPi" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_PI), // italicized
    "varSigma" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_SIGMA), // italicized
    "varTheta" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_THETA), // italicized
    "varUpsilon" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_UPSILON), // italicized
    "varXi" => Token::Letter(symbol::GREEK_CAPITAL_LETTER_XI), // italicized
    "varepsilon" => Token::Letter(symbol::GREEK_SMALL_LETTER_EPSILON),
    "varkappa" => Token::Letter(symbol::GREEK_KAPPA_SYMBOL),
    "varnothing" => Token::Letter(symbol::EMPTY_SET),
    "varointclockwise" => Token::Integral(symbol::CLOCKWISE_CONTOUR_INTEGRAL),
    "varphi" => Token::Letter(symbol::GREEK_SMALL_LETTER_PHI),
    "varpi" => Token::Letter(symbol::GREEK_PI_SYMBOL),
    "varrho" => Token::Letter(symbol::GREEK_RHO_SYMBOL),
    "varsigma" => Token::Letter(symbol::GREEK_SMALL_LETTER_FINAL_SIGMA),
    "vartheta" => Token::Letter(symbol::GREEK_THETA_SYMBOL),
    "vartriangle" => Token::Letter(symbol::WHITE_UP_POINTING_TRIANGLE),
    "vdash" => Token::Relation(symbol::RIGHT_TACK),
    "vdots" => Token::Relation(symbol::VERTICAL_ELLIPSIS),
    "vec" => Token::OverUnder(symbol::COMBINING_RIGHT_ARROW_ABOVE, true, Some(OpAttr::StretchyFalse)),
    "vee" => Token::BinaryOp(symbol::LOGICAL_OR),
    "veebar" => Token::BinaryOp(symbol::XOR),
    "veeeq" => Token::Relation(symbol::EQUIANGULAR_TO), // from "stix"
    "venus" => Token::Letter(symbol::FEMALE_SIGN),
    "vert" => Token::Delimiter(symbol::VERTICAL_LINE),
    "wedge" => Token::BinaryOp(symbol::LOGICAL_AND),
    "wedgeq" => Token::Relation(symbol::ESTIMATES), // from "stix"
    "widehat" => Token::OverUnder(symbol::CIRCUMFLEX_ACCENT, true, None),
    "wideparen" => Token::OverUnderBrace(symbol::TOP_PARENTHESIS, true),
    "widetilde" => Token::OverUnder(symbol::TILDE, true, None),
    "wp" => Token::Letter(symbol::SCRIPT_CAPITAL_P),
    "wr" => Token::Relation(symbol::WREATH_PRODUCT),
    "xi" => Token::Letter(symbol::GREEK_SMALL_LETTER_XI),
    "xleftarrow" => Token::CustomCmd(1, NodeRef::new(&predefined::XLEFTARROW)),
    "xrightarrow" => Token::CustomCmd(1, NodeRef::new(&predefined::XRIGHTARROW)),
    "zeta" => Token::Letter(symbol::GREEK_SMALL_LETTER_ZETA),
    "{" => Token::Delimiter(symbol::LEFT_CURLY_BRACKET),
    "|" => Token::Delimiter(symbol::DOUBLE_VERTICAL_LINE),
    "}" => Token::Delimiter(symbol::RIGHT_CURLY_BRACKET),
};

pub fn get_command(command: &str) -> Token<'_> {
    match COMMANDS.get(command) {
        Some(token) => *token,
        None => Token::UnknownCommand(command),
    }
}

pub fn get_negated_op(op: Rel) -> Option<Rel> {
    match op {
        symbol::ALMOST_EQUAL_TO => Some(symbol::NOT_ALMOST_EQUAL_TO),
        symbol::APPROXIMATELY_EQUAL_TO => Some(symbol::NOT_ASYMPTOTICALLY_EQUAL_TO),
        symbol::DOUBLE_VERTICAL_BAR_DOUBLE_RIGHT_TURNSTILE => {
            Some(symbol::NEGATED_DOUBLE_VERTICAL_BAR_DOUBLE_RIGHT_TURNSTILE)
        }
        symbol::ELEMENT_OF => Some(symbol::NOT_AN_ELEMENT_OF),
        symbol::FORCES => Some(symbol::DOES_NOT_FORCE),
        symbol::GREATER_THAN_OVER_EQUAL_TO => Some(symbol::NEITHER_GREATER_THAN_NOR_EQUAL_TO),
        symbol::LESS_THAN_OR_EQUAL_TO => Some(symbol::NEITHER_LESS_THAN_NOR_EQUAL_TO),
        symbol::PRECEDES => Some(symbol::DOES_NOT_PRECEDE),
        symbol::RIGHT_TACK => Some(symbol::DOES_NOT_PROVE),
        symbol::SUBSET_OF => Some(symbol::NOT_A_SUBSET_OF),
        symbol::SUBSET_OF_OR_EQUAL_TO => Some(symbol::NEITHER_A_SUBSET_OF_NOR_EQUAL_TO),
        symbol::SUCCEEDS => Some(symbol::DOES_NOT_SUCCEED),
        symbol::SUPERSET_OF => Some(symbol::NOT_A_SUPERSET_OF),
        symbol::SUPERSET_OF_OR_EQUAL_TO => Some(symbol::NEITHER_A_SUPERSET_OF_NOR_EQUAL_TO),
        symbol::TRUE => Some(symbol::NOT_TRUE),
        _ => None,
    }
}

static TEXT_COMMANDS: phf::Map<&'static str, Token> = phf::phf_map! {
    "%" => Token::Letter('%'),
    "'" => Token::TextModeAccent(symbol::COMBINING_ACUTE_ACCENT),
    "." => Token::TextModeAccent(symbol::COMBINING_DOT_ABOVE),
    "=" => Token::TextModeAccent(symbol::COMBINING_OVERLINE),
    "AA" => Token::Letter('Å'),
    "AE" => Token::Letter('Æ'),
    "DH" => Token::Letter('Ð'),
    "DJ" => Token::Letter('Ð'),
    "H" => Token::TextModeAccent(symbol::COMBINING_DOUBLE_ACUTE_ACCENT),
    "L" => Token::Letter('Ł'),
    "NG" => Token::Letter('Ŋ'),
    "O" => Token::Letter('Ø'),
    "OE" => Token::Letter('Œ'),
    "TH" => Token::Letter('Þ'),
    "\"" => Token::TextModeAccent(symbol::COMBINING_DIAERESIS),
    "^" => Token::TextModeAccent(symbol::COMBINING_CIRCUMFLEX_ACCENT.as_op().as_char()),
    "`" => Token::TextModeAccent(symbol::COMBINING_GRAVE_ACCENT),
    "aa" => Token::Letter('å'),
    "ae" => Token::Letter('æ'),
    "c" => Token::TextModeAccent(symbol::COMBINING_CEDILLA),
    "dh" => Token::Letter('ð'),
    "dj" => Token::Letter('đ'),
    "dots" => Token::Letter(symbol::HORIZONTAL_ELLIPSIS),
    "i" => Token::Letter('ı'),
    "j" => Token::Letter('ȷ'),
    "l" => Token::Letter('ł'),
    "ldots" => Token::Letter(symbol::HORIZONTAL_ELLIPSIS),
    "ng" => Token::Letter('ŋ'),
    "o" => Token::Letter('ø'),
    "oe" => Token::Letter('œ'),
    "r" => Token::TextModeAccent(symbol::COMBINING_RING_ABOVE),
    "rq" => Token::Letter('’'),
    "ss" => Token::Letter('ß'),
    "text" => Token::Text(None),
    "textbackslash" => Token::Letter('\\'),
    "textbf" => Token::Text(Some(TextTransform::Bold)),
    "textit" => Token::Text(Some(TextTransform::Italic)),
    "textstyle" => Token::Style(Style::Text),
    "texttt" => Token::Text(Some(TextTransform::Monospace)),
    "textyen" => Token::Letter('¥'),
    "u" => Token::TextModeAccent(symbol::COMBINING_BREVE),
    "v" => Token::TextModeAccent(symbol::COMBINING_CARON.as_op().as_char()),
    "{" => Token::Delimiter(symbol::LEFT_CURLY_BRACKET),
    "|" => Token::Delimiter(symbol::DOUBLE_VERTICAL_LINE),
    "}" => Token::Delimiter(symbol::RIGHT_CURLY_BRACKET),
    "~" => Token::TextModeAccent(symbol::COMBINING_TILDE.as_op().as_char()),
};

pub fn get_text_command(command: &str) -> Token<'_> {
    match TEXT_COMMANDS.get(command) {
        Some(token) => *token,
        None => Token::UnknownCommand(command),
    }
}
